@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: curs_main.c,v 2.3 1998/12/31 09:45:46 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -83,8 +82,8 @@ extern const char *ReleaseDate;
 
 void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
 {
-  format_flag flag = M_FORMAT_MAKEPRINT | M_FORMAT_ARROWCURSOR;
-  int reverse = Sort & SORT_REVERSE, edgemsgno;
+  format_flag flag = M_FORMAT_MAKEPRINT | M_FORMAT_ARROWCURSOR | M_FORMAT_INDEX;
+  int edgemsgno, reverse = Sort & SORT_REVERSE;
   HEADER *tmp, *h = Context->hdrs[Context->v2r[num]];
 
   if ((Sort & SORT_MASK) == SORT_THREADS && h->tree)
@@ -97,18 +96,19 @@ void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
     {
       if (reverse)
       {
-       if (menu->top + menu->pagelen > menu->max)
-         edgemsgno = Context->v2r[menu->max - 1];
-       else
-         edgemsgno = Context->v2r[menu->top + menu->pagelen - 1];
+	if (menu->top + menu->pagelen > menu->max)
+	  edgemsgno = Context->v2r[menu->max - 1];
+	else
+	  edgemsgno = Context->v2r[menu->top + menu->pagelen - 1];
       }
       else
-       edgemsgno = Context->v2r[menu->top];
+	edgemsgno = Context->v2r[menu->top];
 
       for (tmp = h->parent; tmp; tmp = tmp->parent)
       {
-	if ((reverse && tmp->msgno > edgemsgno)
-	    || (!reverse && tmp->msgno < edgemsgno))
+	/* if no ancestor is visible on current screen, provisionally force
+	 * subject... */
+	if (reverse ? tmp->msgno > edgemsgno : tmp->msgno < edgemsgno)
 	{
 	  flag |= M_FORMAT_FORCESUBJ;
 	  break;
@@ -120,8 +120,8 @@ void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
       {
 	for (tmp = h->prev; tmp; tmp = tmp->prev)
 	{
-	  if ((reverse && tmp->msgno > edgemsgno)
-	      || (!reverse && tmp->msgno < edgemsgno))
+	  /* ...but if a previous sibling is available, don't force it */
+	  if (reverse ? tmp->msgno > edgemsgno : tmp->msgno < edgemsgno)
 	    break;
 	  else if (tmp->virtual >= 0)
 	  {
@@ -133,7 +133,7 @@ void index_make_entry (char *s, size_t l, MUTTMENU *menu, int num)
     }
   }
 
-  _mutt_make_string (s, l, NONULL (HdrFmt), Context, h, flag | M_FORMAT_INDEX);
+  _mutt_make_string (s, l, NONULL (HdrFmt), Context, h, flag);
 }
 
 int index_color (int index_no)
@@ -321,7 +321,8 @@ int mutt_index_menu (void)
       }
       else if (check == M_NEW_MAIL || check == M_REOPENED)
       {
-	int *save_new;
+	/* store pointers to the newly added messages */
+	HEADER  **save_new = NULL;
 
 	/* take note of the current message */
 	if (oldcount)
@@ -358,15 +359,13 @@ int mutt_index_menu (void)
 	/* save the list of new messages */
 	if (oldcount && check != M_REOPENED)
 	{
-	  save_new = (int *) safe_malloc (sizeof (int) * (Context->msgcount - oldcount));
+	  save_new = (HEADER **) safe_malloc (sizeof (HEADER *) * (Context->msgcount - oldcount));
 	  for (j = oldcount; j < Context->msgcount; j++)
-	    save_new[j-oldcount] = Context->hdrs[j]->index;
+	    save_new[j-oldcount] = Context->hdrs[j];
         }
 
 	/* if the mailbox was reopened, need to rethread from scratch */
-	set_option (OPTSORTCOLLAPSE);
 	mutt_sort_headers (Context, (check == M_REOPENED));
-	unset_option (OPTSORTCOLLAPSE);
 
 	/* uncollapse threads with new mail */
 	if ((Sort & SORT_MASK) == SORT_THREADS)
@@ -394,7 +393,7 @@ int mutt_index_menu (void)
 	      for (k = 0; k < Context->msgcount; k++)
 	      {
 		HEADER *h = Context->hdrs[k];
-		if (h->index == save_new[j] && (!Context->pattern || h->limited))
+		if (h == save_new[j] && (!Context->pattern || h->limited))
 		  mutt_uncollapse_thread (Context, h);
 	      }
 	    }
@@ -1140,7 +1139,14 @@ int mutt_index_menu (void)
 
 	  if (CUR->collapsed)
 	  {
-	    if (op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD || UNREAD(CUR) == 1)
+	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
+		UNREAD (CUR))
+	    {
+	      menu->current = i;
+	      break;
+	    }
+	    if ((op == OP_MAIN_NEXT_NEW || op == OP_MAIN_PREV_NEW) &&
+		UNREAD (CUR) == 1)
 	    {
 	      menu->current = i;
 	      break;
