@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: mbox.c,v 2.0 1998/12/12 08:06:39 roessler Rel $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -54,13 +53,14 @@ int mbox_lock_mailbox (CONTEXT *ctx, int excl, int retry)
 {
   int r;
 
-  /* XXX - Currently, we force dotlocking.
-   * Use dotlock -t here.
-   */
-
   if ((r = mx_lock_file (ctx->path, fileno (ctx->fp), excl, 1, retry)) == 0)
     ctx->locked = 1;
-
+  else if (retry && !excl)
+  {
+    ctx->readonly = 1;
+    return 0;
+  }
+  
   return (r);
 }
 
@@ -107,7 +107,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 
   /* precompute the local timezone to speed up calculation of the
      received time */
-  tz = mutt_local_tz ();
+  tz = mutt_local_tz (0);
 
   buf[sizeof (buf) - 1] = 0;
   
@@ -145,7 +145,7 @@ int mmdf_parse_mailbox (CONTEXT *ctx)
 	}
       } 
       else
-	hdr->received = t + tz;
+	hdr->received = t - tz;
 
       hdr->env = mutt_read_rfc822_header (ctx->fp, hdr, 0);
 
@@ -249,7 +249,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 
   /* precompute the local timezone to speed up calculation of the
      date received */
-  tz = mutt_local_tz ();
+  tz = mutt_local_tz (0);
 
   loc = ftell (ctx->fp);
   while (fgets (buf, sizeof (buf), ctx->fp) != NULL)
@@ -281,7 +281,7 @@ int mbox_parse_mailbox (CONTEXT *ctx)
 	mx_alloc_memory (ctx);
       
       curhdr = ctx->hdrs[ctx->msgcount] = mutt_new_header ();
-      curhdr->received = t + tz;
+      curhdr->received = t - tz;
       curhdr->offset = loc;
       curhdr->index = ctx->msgcount;
 	
@@ -576,7 +576,7 @@ int mbox_check_mailbox (CONTEXT *ctx, int *index_hint)
 	  /* we couldn't lock the mailbox, but nothing serious happened:
 	   * probably the new mail arrived: no reason to wait till we can
 	   * parse it: we'll get it on the next pass
-	   *  */
+	   */
 	  return (M_LOCKED);
 	}
 	unlock = 1;
@@ -673,9 +673,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
   {
     save_sort = Sort;
     Sort = SORT_ORDER;
-    set_option (OPTSORTCOLLAPSE);
     mutt_sort_headers (ctx, 0);
-    unset_option (OPTSORTCOLLAPSE);
   }
 
   /* need to open the file for writing in such a way that it does not truncate
@@ -717,7 +715,10 @@ int mbox_sync_mailbox (CONTEXT *ctx)
       (fp = fdopen (i, "w")) == NULL)
   {
     if (-1 != i)
+    {
       close (i);
+      unlink (tempfile);
+    }
     mutt_error _("Could not create temporary file!");
     goto bail;
   }
@@ -737,6 +738,7 @@ int mbox_sync_mailbox (CONTEXT *ctx)
     mutt_error _("sync: mbox modified, but no modified messages! (report this bug)");
     sleep(5); /* the mutt_error /will/ get cleared! */
     dprint(1, (debugfile, "mbox_sync_mailbox(): no modified messages.\n"));
+    unlink (tempfile);
     goto bail;
   }
 
@@ -749,9 +751,9 @@ int mbox_sync_mailbox (CONTEXT *ctx)
    * sure we seek to the correct location
    */
   if (ctx->magic == M_MMDF)
-    offset -= mutt_strlen (MMDF_SEP);
+    offset -= (sizeof MMDF_SEP - 1);
   else if (ctx->magic == M_KENDRA)
-    offset -= mutt_strlen(KENDRA_SEP);
+    offset -= (sizeof KENDRA_SEP - 1);
   
   /* allocate space for the new offsets */
   newOffset = safe_calloc (ctx->msgcount - first, sizeof (struct m_update_t));
@@ -768,12 +770,19 @@ int mbox_sync_mailbox (CONTEXT *ctx)
       if (ctx->magic == M_MMDF)
       {
 	if (fputs (MMDF_SEP, fp) == EOF)
+	{
+	  unlink (tempfile);
 	  goto bail;
+	}
+	  
       }
       else if (ctx->magic == M_KENDRA)
       {
 	if (fputs (KENDRA_SEP, fp) == EOF)
+	{
+	  unlink (tempfile);
 	  goto bail;
+	}
       }
 
       /* save the new offset for this message.  we add `offset' because the
@@ -783,7 +792,10 @@ int mbox_sync_mailbox (CONTEXT *ctx)
       newOffset[i - first].hdr = ftell (fp) + offset;
 
       if (mutt_copy_message (fp, ctx, ctx->hdrs[i], M_CM_UPDATE, CH_FROM | CH_UPDATE | CH_UPDATE_LEN) == -1)
+      {
+	unlink (tempfile);
 	goto bail;
+      }
 
       /* Since messages could have been deleted, the offsets stored in memory
        * will be wrong, so update what we can, which is the offset of this
@@ -797,13 +809,25 @@ int mbox_sync_mailbox (CONTEXT *ctx)
       switch(ctx->magic)
       {
 	case M_MMDF: 
-	  if(fputs(MMDF_SEP, fp) == EOF) goto bail; 
+	  if(fputs(MMDF_SEP, fp) == EOF) 
+	  {
+	    unlink (tempfile);
+	    goto bail; 
+	  }
 	  break;
 	case M_KENDRA:
-	  if(fputs(KENDRA_SEP, fp) == EOF) goto bail;
+	  if(fputs(KENDRA_SEP, fp) == EOF)
+	  {
+	    unlink (tempfile);
+	    goto bail;
+	  }
 	  break;
 	default:
-	  if(fputs("\n", fp) == EOF) goto bail;
+	  if(fputs("\n", fp) == EOF) 
+	  {
+	    unlink (tempfile);
+	    goto bail;
+	  }
       }
     }
   }
@@ -948,9 +972,7 @@ bail:  /* Come here in case of disaster */
     Sort = save_sort;
     /* if the mailbox was reopened, the thread tree will be invalid so make
      * sure to start threading from scratch.  */
-    set_option (OPTSORTCOLLAPSE);
     mutt_sort_headers (ctx, (need_sort == M_REOPENED));
-    unset_option (OPTSORTCOLLAPSE);
   }
 
   return (-1);
@@ -1021,8 +1043,8 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
   ctx->unread = 0;
   ctx->flagged = 0;
   ctx->changed = 0;
-  ctx->id_hash = hash_create (257);
-  ctx->subj_hash = hash_create (257);
+  ctx->id_hash = hash_create (1031);
+  ctx->subj_hash = hash_create (1031);
 
   switch (ctx->magic)
   {
